import LatexJS from "../Latex.js";

export class SkribblWord extends HTMLElement {
	/**
	 * @param {string} word
	 * @param {string} description
	 * @param {string[]} synonyms
	 */
	constructor(word="",description="",synonyms=[]){
		super();
		this.attachShadow({mode:"open"});
		this.shadowRoot.innerHTML = `
			<style>
				:host {
					display: block;
				}
				latex-js {
					display: block;
					padding-left: 20px;
					padding-right: 20px;
				}
				h3 {
					margin-bottom: 10px;
					margin-top: 18px;
				}
			</style>
			<h3></h3>
		`;
		this._word = this.shadowRoot.querySelector("h3");
		this._description = new LatexJS(description,{delay:true});
		this.shadowRoot.appendChild(this._description);
		this.word = word;
		this.synonyms = synonyms;
	}

	set word(word){
		this.setAttribute("word",word);
	}

	get word(){
		return this.getAttribute("word");
	}

	set description(description){
		this.setAttribute("description",description);
	}

	get description(){
		return this.getAttribute("description");
	}

	set synonyms(synonyms){
		if (synonyms&&synonyms.length>0){
			this.setAttribute("synonyms",synonyms.join(", "));
		}else{
			this.removeAttribute("synonyms");
		}
	}

	get synonyms(){
		let s = this.getAttribute("synonyms");
		return s?s.split(",").map(s=>s.trim()):[];
	}

	attributeChangedCallback(name,oldValue,newValue){
		if (name=="word"||name=="synonyms"){
			this.refreshTitle();
		}else if(name=="description"){
			this._description.latex = newValue;
		}
	}

	refreshTitle(){
		this._word.textContent = [this.word,...this.synonyms||[]].join(", ");
	}

	static get observedAttributes(){
		return ["word","description", "synonyms"];
	}
}
export class SkribblTopic extends HTMLElement {
	constructor(name=""){
		super();
		this.attachShadow({mode:"open"});
		this.shadowRoot.innerHTML = `
			<style>
				:host {
					display: block;
					margin-bottom: 50px;
				}
				h2 {
					text-align: center;
				}
			</style>
			<h2></h2>
			<slot></slot>
		`;
		this._name = this.shadowRoot.querySelector("h2");
		this.name = name;
	}

	set name(name){
		this.setAttribute("name",name);
	}

	get name(){
		return this.getAttribute("name");
	}

	attributeChangedCallback(name,oldValue,newValue){
		if (name=="name"){
			this._name.textContent = newValue;
			this.id = newValue.replace(/ /g,"-");
		}
	}

	static get observedAttributes(){
		return ["name"];
	}
}
customElements.define("skribbl-word",SkribblWord);
customElements.define("skribbl-topic",SkribblTopic);